urcu-bp: introduce struct urcu_gp
authorMathieu Desnoyers <mathieu.desnoyers@efficios.com>
Mon, 6 May 2013 14:30:57 +0000 (10:30 -0400)
committerMathieu Desnoyers <mathieu.desnoyers@efficios.com>
Mon, 6 May 2013 14:30:57 +0000 (10:30 -0400)
Make urcu-bp similar to urcu-qsbr and other urcu flavors.

Signed-off-by: Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
urcu-bp.c
urcu/map/urcu-bp.h
urcu/static/urcu-bp.h

index 8207a4120ffd6b0790939e0ee176e064198704fe..ef1e68771ab9fafe8b67da4c98672f24edd43006 100644 (file)
--- a/urcu-bp.c
+++ b/urcu-bp.c
@@ -109,13 +109,7 @@ unsigned int rcu_yield_active;
 DEFINE_URCU_TLS(unsigned int, rcu_rand_yield);
 #endif
 
-/*
- * Global grace period counter.
- * Contains the current RCU_GP_CTR_PHASE.
- * Also has a RCU_GP_COUNT of 1, to accelerate the reader fast path.
- * Written to only by writer with mutex taken. Read by both writer and readers.
- */
-unsigned long rcu_gp_ctr = RCU_GP_COUNT;
+struct rcu_gp rcu_gp = { .ctr = RCU_GP_COUNT };
 
 /*
  * Pointer to registry elements. Written to only by each individual reader. Read
@@ -174,7 +168,7 @@ static void wait_for_readers(struct cds_list_head *input_readers,
        /*
         * Wait for each thread URCU_TLS(rcu_reader).ctr to either
         * indicate quiescence (not nested), or observe the current
-        * rcu_gp_ctr value.
+        * rcu_gp.ctr value.
         */
        for (;;) {
                wait_loops++;
@@ -250,7 +244,7 @@ void synchronize_rcu(void)
        cmm_smp_mb();
 
        /* Switch parity: 0 -> 1, 1 -> 0 */
-       CMM_STORE_SHARED(rcu_gp_ctr, rcu_gp_ctr ^ RCU_GP_CTR_PHASE);
+       CMM_STORE_SHARED(rcu_gp.ctr, rcu_gp.ctr ^ RCU_GP_CTR_PHASE);
 
        /*
         * Must commit qparity update to memory before waiting for other parity
index 8c819a68afaf3902a71c83f20d6a972c6899305a..92863fef257dfd3d5757c855a5afdf98cfa3a197 100644 (file)
@@ -46,8 +46,7 @@
 #define rcu_exit                       rcu_exit_bp
 #define synchronize_rcu                        synchronize_rcu_bp
 #define rcu_reader                     rcu_reader_bp
-#define rcu_gp_ctr                     rcu_gp_ctr_bp
-#define rcu_gp_futex                   rcu_gp_futex_bp /* unused */
+#define rcu_gp                         rcu_gp_bp
 
 #define get_cpu_call_rcu_data          get_cpu_call_rcu_data_bp
 #define get_call_rcu_thread            get_call_rcu_thread_bp
index 2f365206afad6db4c561654109def617ef559ce5..1052c44fad962efa7bb3b9781c3b6097a39257ac 100644 (file)
@@ -130,12 +130,18 @@ static inline void rcu_debug_yield_init(void)
  */
 extern void rcu_bp_register(void);
 
-/*
- * Global quiescent period counter with low-order bits unused.
- * Using a int rather than a char to eliminate false register dependencies
- * causing stalls on some architectures.
- */
-extern unsigned long rcu_gp_ctr;
+struct rcu_gp {
+       /*
+        * Global grace period counter.
+        * Contains the current RCU_GP_CTR_PHASE.
+        * Also has a RCU_GP_COUNT of 1, to accelerate the reader fast path.
+        * Written to only by writer with mutex taken.
+        * Read by both writer and readers.
+        */
+       unsigned long ctr;
+} __attribute__((aligned(CAA_CACHE_LINE_SIZE)));
+
+extern struct rcu_gp rcu_gp;
 
 struct rcu_reader {
        /* Data used by both reader and synchronize_rcu() */
@@ -166,13 +172,13 @@ static inline enum rcu_state rcu_reader_state(unsigned long *ctr)
        v = CMM_LOAD_SHARED(*ctr);
        if (!(v & RCU_GP_CTR_NEST_MASK))
                return RCU_READER_INACTIVE;
-       if (!((v ^ rcu_gp_ctr) & RCU_GP_CTR_PHASE))
+       if (!((v ^ rcu_gp.ctr) & RCU_GP_CTR_PHASE))
                return RCU_READER_ACTIVE_CURRENT;
        return RCU_READER_ACTIVE_OLD;
 }
 
 /*
* Helper for _rcu_read_lock().  The format of rcu_gp_ctr (as well as
  * the per-thread rcu_reader.ctr) has the upper bits containing a count of
  * _rcu_read_lock() nesting, and a lower-order bit that contains either zero
  * or RCU_GP_CTR_PHASE.  The smp_mb_slave() ensures that the accesses in
@@ -181,7 +187,7 @@ static inline enum rcu_state rcu_reader_state(unsigned long *ctr)
 static inline void _rcu_read_lock_update(unsigned long tmp)
 {
        if (caa_likely(!(tmp & RCU_GP_CTR_NEST_MASK))) {
-               _CMM_STORE_SHARED(URCU_TLS(rcu_reader)->ctr, _CMM_LOAD_SHARED(rcu_gp_ctr));
+               _CMM_STORE_SHARED(URCU_TLS(rcu_reader)->ctr, _CMM_LOAD_SHARED(rcu_gp.ctr));
                cmm_smp_mb();
        } else
                _CMM_STORE_SHARED(URCU_TLS(rcu_reader)->ctr, tmp + RCU_GP_COUNT);
@@ -166,13 +172,13 @@ static inline enum rcu_state rcu_reader_state(unsigned long *ctr)
 
 /*
- * Helper for _rcu_read_lock().  The format of rcu_gp_ctr (as well as
+ * Helper for _rcu_read_lock().  The format of rcu_gp.ctr (as well as
  * the per-thread rcu_reader.ctr) has the upper bits containing a count of
  * _rcu_read_lock() nesting, and a lower-order bit that contains either zero
This page took 0.027065 seconds and 4 git commands to generate.